@@ -441,6 +441,17 @@ fill_out_screen_info (Display *xdisplay,
 					 &(info->max_height));
     }
 
+    info->primary = None;
+#if (RANDR_MAJOR > 1 || (RANDR_MAJOR == 1 && RANDR_MINOR >= 3))
+    /* Runtime check for RandR 1.3 or higher */
+    if (info->screen->rr_major_version == 1 && info->screen->rr_minor_version >= 3) {
+        gdk_error_trap_push ();
+        info->primary = XRRGetOutputPrimary (xdisplay, xroot);
+	gdk_flush ();
+	gdk_error_trap_pop (); /* ignore error */
+    }
+#endif
+
     return TRUE;
 }
 
@@ -1129,6 +1140,30 @@ gnome_rr_output_can_clone (GnomeRROutput *output,
     return FALSE;
 }
 
+gboolean
+gnome_rr_output_get_is_primary (GnomeRROutput *output)
+{
+    return output->info->primary == output->id;
+}
+
+void
+gnome_rr_screen_set_primary_output (GnomeRRScreen *screen,
+                                    GnomeRROutput *output)
+{
+#if (RANDR_MAJOR > 1 || (RANDR_MAJOR == 1 && RANDR_MINOR >= 3))
+    RROutput id;
+
+    if (output)
+        id = output->id;
+    else
+        id = None;
+
+        /* Runtime check for RandR 1.3 or higher */
+    if (screen->rr_major_version == 1 && screen->rr_minor_version >= 3)
+        XRRSetOutputPrimary (screen->xdisplay, screen->xroot, id);
+#endif
+}
+
 /* GnomeRRCrtc */
 typedef struct
 {
@@ -1176,6 +1211,7 @@ xrotation_from_rotation (GnomeRRRotation r)
     return result;
 }
 
+#ifndef GNOME_DISABLE_DEPRECATED_SOURCE
 gboolean
 gnome_rr_crtc_set_config (GnomeRRCrtc      *crtc,
 			  int               x,
@@ -1188,6 +1224,7 @@ gnome_rr_crtc_set_config (GnomeRRCrtc      *crtc,
 {
     return gnome_rr_crtc_set_config_with_time (crtc, GDK_CURRENT_TIME, x, y, mode, rotation, outputs, n_outputs, error);
 }
+#endif
 
 gboolean
 gnome_rr_crtc_set_config_with_time (GnomeRRCrtc      *crtc,
