@@ -48,14 +48,31 @@ typedef enum
     GNOME_RR_REFLECT_Y =	(1 << 5)
 } GnomeRRRotation;
 
+/* Error codes */
+
+#define GNOME_RR_ERROR (gnome_rr_error_quark ())
+
+GQuark gnome_rr_error_quark (void);
+
+typedef enum {
+    GNOME_RR_ERROR_UNKNOWN,		/* generic "fail" */
+    GNOME_RR_ERROR_NO_RANDR_EXTENSION,	/* RANDR extension is not present */
+    GNOME_RR_ERROR_RANDR_ERROR,		/* generic/undescribed error from the underlying XRR API */
+    GNOME_RR_ERROR_BOUNDS_ERROR,	/* requested bounds of a CRTC are outside the maximum size */
+    GNOME_RR_ERROR_CRTC_ASSIGNMENT,	/* could not assign CRTCs to outputs */
+    GNOME_RR_ERROR_NO_MATCHING_CONFIG,	/* none of the saved configurations matched the current configuration */
+} GnomeRRError;
+
 /* GnomeRRScreen */
 GnomeRRScreen * gnome_rr_screen_new                (GdkScreen             *screen,
 						    GnomeRRScreenChanged   callback,
-						    gpointer               data);
+						    gpointer               data,
+						    GError               **error);
 void            gnome_rr_screen_destroy            (GnomeRRScreen         *screen);
 GnomeRROutput **gnome_rr_screen_list_outputs       (GnomeRRScreen         *screen);
 GnomeRRCrtc **  gnome_rr_screen_list_crtcs         (GnomeRRScreen         *screen);
 GnomeRRMode **  gnome_rr_screen_list_modes         (GnomeRRScreen         *screen);
+GnomeRRMode **  gnome_rr_screen_list_clone_modes   (GnomeRRScreen	  *screen);
 void            gnome_rr_screen_set_size           (GnomeRRScreen         *screen,
 						    int                    width,
 						    int                    height,
@@ -63,7 +80,8 @@ void            gnome_rr_screen_set_size           (GnomeRRScreen         *scree
 						    int                    mm_height);
 GnomeRRCrtc *   gnome_rr_screen_get_crtc_by_id     (GnomeRRScreen         *screen,
 						    guint32                id);
-gboolean        gnome_rr_screen_refresh            (GnomeRRScreen         *screen);
+gboolean        gnome_rr_screen_refresh            (GnomeRRScreen         *screen,
+						    GError               **error);
 GnomeRROutput * gnome_rr_screen_get_output_by_id   (GnomeRRScreen         *screen,
 						    guint32                id);
 GnomeRROutput * gnome_rr_screen_get_output_by_name (GnomeRRScreen         *screen,
@@ -73,6 +91,12 @@ void            gnome_rr_screen_get_ranges         (GnomeRRScreen         *scree
 						    int                   *max_width,
 						    int                   *min_height,
 						    int                   *max_height);
+void            gnome_rr_screen_get_timestamps     (GnomeRRScreen         *screen,
+						    guint32               *change_timestamp_ret,
+						    guint32               *config_timestamp_ret);
+
+void            gnome_rr_screen_set_primary_output (GnomeRRScreen         *screen,
+                                                    GnomeRROutput         *output);
 
 /* GnomeRROutput */
 guint32         gnome_rr_output_get_id             (GnomeRROutput         *output);
@@ -94,6 +118,7 @@ GnomeRRMode **  gnome_rr_output_list_modes         (GnomeRROutput         *outpu
 GnomeRRMode *   gnome_rr_output_get_preferred_mode (GnomeRROutput         *output);
 gboolean        gnome_rr_output_supports_mode      (GnomeRROutput         *output,
 						    GnomeRRMode           *mode);
+gboolean        gnome_rr_output_get_is_primary     (GnomeRROutput         *output);
 
 /* GnomeRRMode */
 guint32         gnome_rr_mode_get_id               (GnomeRRMode           *mode);
@@ -103,13 +128,27 @@ int             gnome_rr_mode_get_freq             (GnomeRRMode           *mode)
 
 /* GnomeRRCrtc */
 guint32         gnome_rr_crtc_get_id               (GnomeRRCrtc           *crtc);
+
+#ifndef GNOME_DISABLE_DEPRECATED
 gboolean        gnome_rr_crtc_set_config           (GnomeRRCrtc           *crtc,
 						    int                    x,
 						    int                    y,
 						    GnomeRRMode           *mode,
 						    GnomeRRRotation        rotation,
 						    GnomeRROutput        **outputs,
-						    int                    n_outputs);
+						    int                    n_outputs,
+						    GError               **error);
+#endif
+
+gboolean        gnome_rr_crtc_set_config_with_time (GnomeRRCrtc           *crtc,
+						    guint32                timestamp,
+						    int                    x,
+						    int                    y,
+						    GnomeRRMode           *mode,
+						    GnomeRRRotation        rotation,
+						    GnomeRROutput        **outputs,
+						    int                    n_outputs,
+						    GError               **error);
 gboolean        gnome_rr_crtc_can_drive_output     (GnomeRRCrtc           *crtc,
 						    GnomeRROutput         *output);
 GnomeRRMode *   gnome_rr_crtc_get_current_mode     (GnomeRRCrtc           *crtc);
